#!/usr/bin/env bash

# # shellcheck disable=SC2154,SC1091,SC2155
# /scripts/gen-user-secret.sh
#
# This helper script will handle user setup of the following users:
#  - ${namingPrefix}-user
#    - password found in ${dbUserCredsSecretPassName}
#    - username found in "${namingPrefix}-${username}-pg-user"
#  OR
#  - a list of users created from a user-supplied variable: ${dbOverrideUserList}
# ----
#  - ${adminUsername} doesn't get created because it's either done via Terraform or via the pod
#    Environment Variables.
#    - password found in ${dbPrimaryCredentialsSecretName} in Infra Namespace only
#

## some of these variables are assumed from environment variables in the inputs-in-env ConfigMap:
# adminUsername
# appsNamespace
# awsDeployment
# databaseName
# dbOverrideUserList
# dbServiceClusterEndpoint
# dbUserCredsSecretPassName
# dbUserCredsSecretUserName
# defaultSchema
# envCategory
# infraNamespace
# namingPrefix
# partialInstallInfra

## There's a dependency on the 'aurora.dbOverrideUserList' Helm helper here.
## To date, if a user supplies the Helm value '.dbOverrideUserList', it will have
## a different structure than if they do not. yay. Need to account for that.
## The structure here are keys/values for a k8s config map that is commonly
## injected as Environment Variables.
## This is the structure around which this script accounts:
##
## {{- if .Values.dbOverrideUserList -}}
##   dbOverrideUserList: {{ .Values.dbOverrideUserList | quote }}
## {{- else -}}
##   dbUserCredsSecretPassName: {{ include "aurora._dbUserPassCredentialsSecretName" . }}
##   dbUserCredsSecretUserName: {{ include "aurora._dbUserUsernameCredentialsSecretName" . }}
##
## where 'dbOverrideUserList' seems to want a list of database users and the other two
## are strings used for the k8s Secret name where the non-Admin 'user' and 'password' secrets are stored.
## So, this helper seems to fulfill two workflows.
## One main difference is that the 'default non-User' Secret name looks like "${namingPrefix}-pg-{user,pass}"
## and this structure for a user-supplied username injects ${username} in the middle:
##  "${namingPrefix}-${username}-pg-{user,pass}"
##

set -eo pipefail

source /scripts/utils.sh

check_access ${infraNamespace} secrets
check_access ${appsNamespace} secrets

## The optional user input is intended as a list of users, so we need to account for a loop.
## condition: if empty string / aka Value not supplied by user
if [[ -z "${dbOverrideUserList}" ]]; then
  ## need an array of 1
  declare -a userList
  userList+=("${namingPrefix}-user")
else
  ## This user input is simply quoted. I'm not sure how that was ever loop-able. But leaving
  ## be for reverse compatibility.
  userList="${dbOverrideUserList}"
fi

echo "UserList is: $userList"

# Set timeout for psql cmd looping: 120s for postgres. 0 secs for aws mode ( no checking loop. execute once only)
if [[ "${awsDeployment}" == "true" ]]; then
  if [[ ("${envCategory}" != "partial") || ("${partialInstallInfra}" == "true") ]]; then
    ckTimeout=0
  else
    ckTimeout=120
  fi
else
   ckTimeout=120
fi

for username in "${userList[@]}"; do
  if [[ -n "${dbOverrideUserList}" ]]; then
    pwd_secret_name="${namingPrefix}-${username}-pg-pass"
    usr_secret_name="${namingPrefix}-${username}-pg-user"
    ## If the user supplies a list of usernames, why would we modify that string? Getting rid of this.
    # username="${namingPrefix}-${username}-user"
  else
    ## via 'aurora.dbOverrideUserList' helper
    pwd_secret_name="${dbUserCredsSecretPassName}"
    usr_secret_name="${dbUserCredsSecretUserName}"
  fi

  ## Assumes that if this user-specific secret exists (which never gets cleaned up) exists then
  ##  there is no need to run any database configuration. This logic is flawed and needs to be revisited.
  cprint "Checking for secret.."
  if /scripts/wait-until -t 30 -i 5 true kubectl --namespace "${appsNamespace}" get secret "${pwd_secret_name}"; then
    cprint "Secret ${pwd_secret_name} already exists."
  else
    cprint "No secret found for ${pwd_secret_name}.. Generating new account."
    cprint "Adding user ${username} to ${dbServiceClusterEndpoint}"
    ## randomly generated value every time
    password=$(gen_password)
    # Change SQL Query for AWS vs Localstack
    if [[ ${awsDeployment} == "true" ]]; then
      SQL=$(cat <<-END
SET password_encryption = 'scram-sha-256';
CREATE USER "${username}" WITH PASSWORD '${password}';
ALTER USER "${username}" WITH PASSWORD '${password}';
GRANT ALL PRIVILEGES ON DATABASE "${databaseName}" TO "${username}";
GRANT rds_replication TO "${username}";
CREATE SCHEMA IF NOT EXISTS "${defaultSchema}";
GRANT ALL PRIVILEGES ON SCHEMA "${defaultSchema}" TO "${username}";
GRANT ALL PRIVILEGES ON ALL TABLES IN SCHEMA "${defaultSchema}" TO "${username}";
GRANT ALL PRIVILEGES ON ALL TABLES IN SCHEMA "${defaultSchema}" TO "${adminUsername}";
CREATE ROLE rds_pgaudit;
CREATE EXTENSION pgaudit;
ALTER DATABASE "${databaseName}" SET pgaudit.log='All';
END
)
    else
      SQL=$(cat <<-END
SET password_encryption = 'scram-sha-256';
CREATE USER "${username}" WITH PASSWORD '${password}';
GRANT ALL PRIVILEGES ON DATABASE "${databaseName}" TO "${username}";
CREATE SCHEMA IF NOT EXISTS "${defaultSchema}";
GRANT ALL PRIVILEGES ON SCHEMA "${defaultSchema}" TO "${username}";
GRANT ALL PRIVILEGES ON ALL TABLES IN SCHEMA "${defaultSchema}" TO "${username}";
END
)
    fi
    cat >/tmp/data.sql <<EOL
${SQL}
EOL
    ## need to run the scripts as $adminUsername only; don't know why this is executed for every user. Might be an opportunity to break this up into
    ##  user-specific sql queries.
    ## ${adminUsername} password ('aurora.dbPrimaryCredentialsSecretName') mounted as Env variable: PGPASSWORD
    /scripts/wait-until -t "${ckTimeout}" -i 5 true psql -h "${dbServiceClusterEndpoint}" -p ${port} -U ${adminUsername} -d "${databaseName}" -f /tmp/data.sql

    ## Create the k8s Secrets for this user's user & password.
    echo "Creating kubernetes secret (user) ${usr_secret_name}"
    /scripts/wait-until -t 30 -i 5 true kubectl --namespace "${infraNamespace}" delete secret "${usr_secret_name}" --ignore-not-found
    /scripts/wait-until -t 30 -i 5 true kubectl --namespace "${infraNamespace}" create secret generic "${usr_secret_name}" --from-literal=user="${username}"
    /scripts/wait-until -t 30 -i 5 true kubectl --namespace "${appsNamespace}" create secret generic "${usr_secret_name}" --from-literal=user="${username}"

    echo "Creating kubernetes secret (password) ${pwd_secret_name}"
    /scripts/wait-until -t 30 -i 5 true kubectl --namespace "${appsNamespace}" create secret generic "${pwd_secret_name}" --from-literal=password="${password}"
    /scripts/wait-until -t 30 -i 5 true kubectl --namespace "${infraNamespace}" create secret generic "${pwd_secret_name}" --from-literal=password="${password}"
  fi
done

print_header "$0 done."
