#!/usr/bin/env bash

# # shellcheck disable=SC2154,SC1091,SC2155
# This helper script will handle a reset of a secret in both the app and infra namespaces
#  so that the generated user's secret is encrypted using the scram-sha-256 algorithm.
#
# /scripts/reset-user-secret-scram.sh
#

## some of these variables are assumed from environment variables in the inputs-in-env ConfigMap:
# adminUsername
# appsNamespace
# databaseName
# dbOverrideUserList
# dbPrimaryCredsSecretName
# dbServiceClusterEndpoint
# dbUserCredsSecretPassName
# infraNamespace
# namingPrefix

## -u can't be used here because some variables are intentionally unset
set -eo pipefail
# set -x

source /scripts/utils.sh

psql_set_password(){
    ## normally I would use lowercase for locals, but I need to make sure these aren't set elsewhere
    local USERNAME=$1; shift
    local PASSWORD=$1; shift
    cat << EOF > /tmp/data.sql
SET password_encryption = 'scram-sha-256';
ALTER USER "${USERNAME}" WITH PASSWORD '${PASSWORD}';
EOF
    cprint "Updating ${USERNAME} to use scram-sha-256 encryption using existing password"
    ## there's an implied assumption that this OS isn't running on a FIPS-enabled OS and can connect with either SCRAM or MD5 authentication
    ## noting this here because this job runs on the common image that this chart produces and is affected by the Dockerfile logic.
    /scripts/wait-until -t 30 -i 5 true psql -h "${dbServiceClusterEndpoint}" -p ${port} -U ${adminUsername} -d "${databaseName}" -f /tmp/data.sql
}

check_access ${infraNamespace} secrets
check_access ${appsNamespace} secrets

if [[ -z "${dbOverrideUserList}" ]]; then
  declare -a userList
  userList+=("${namingPrefix}-user")
  userList+=(${adminUsername})
else
  userList="${dbOverrideUserList}"
fi

echo "UserList is: $userList"

# Set timeout for psql cmd looping: 120s for postgres. 0 secs for aws mode ( no checking loop. execute once only)
if [[ "${awsDeployment}" == "true" ]]; then
  if [[ ("${envCategory}" != "partial") || ("${partialInstallInfra}" == "true") ]]; then
    ckTimeout=0
  else
    ckTimeout=120
  fi
else
   ckTimeout=120
fi

for username in "${userList[@]}"; do
  if [[ ${username} == ${adminUsername} ]]; then
    namespace="${infraNamespace}"
    pwd_secret_name="${dbPrimaryCredsSecretName}"
  else
    namespace="${appsNamespace}"
    ## via 'aurora.dbOverrideUserList' helper
    pwd_secret_name="${dbUserCredsSecretPassName}"
  fi
  cprint "Checking for password for user, ${username}, in namespace ${namespace} ..."

  if /scripts/wait-until -t 30 -i 5 true kubectl --namespace "${namespace}" get secret "${pwd_secret_name}"; then
    password=$(kubectl --namespace "${namespace}" get secret ${pwd_secret_name} -o jsonpath={.data.password} | base64 -d)
    if [[ -z "${password:+x}" ]]; then
        cprint "Secret exists, but unable to retrieve password from ${pwd_secret_name}[.data.password]."
    else
        cprint "DB Password Secret (${pwd_secret_name}[.data.password]) located."
        psql_set_password $username $password
    fi
  else
    cprint "No secret found for ${pwd_secret_name}."
  fi
done

print_header "$0 done."
